;
; pico.s
;

.include "common.inc"

PROFILE = 0

CHR_BANK = $7C
NAM_BANK = $7E
TRACK_COUNT = 10
.exportzp TRACK_COUNT

.feature force_range ; using signed operations, disable ca65's unsigned range check

.import gamepad_poll ; from player.s
.import track_order ; from player.s
.export tick_screen ; for player.s
.export render_off ; for player.s

.segment "CODE"

;
; graphics loading
;

wait_nmi:
	lda nmi_count
	:
		cmp nmi_count
	beq :-
	rts    

render_off:
	jsr wait_nmi
	lda #0
	sta $2000
	sta $2001
	rts

render_on:
	lda #1
	sta gfx_ready
	lda $2002
	lda #%10001000
	sta $2000
	jmp wait_nmi

render_tick:
	.if PROFILE
		lda #%00011111
		sta $2001
	.endif
	lda #1
	sta gfx_ready
	jmp wait_nmi

render_fade_prepare:
	; use gfx_nmt as temporary storage
	ldx #0
	stx gfx_len
	:
		lda gfx_pal, X
		sta gfx_nmt, X
		inx
		cpx #32
		bcc :-
	rts

render_fade_finish:
	ldx #0
	:
		lda gfx_nmt, X
		sta gfx_pal, X
		inx
		cpx #32
		bcc :-
	rts

; param_a = subtract value
render_fade_step:
	ldx #0
	@loop:
		lda gfx_nmt, X
		sec
		sbc param_a
		bpl :+
			lda #$0F
		:
		sta gfx_pal, X
		inx
		cpx #32
		bcc @loop
	rts

render_fade_out:
	jsr render_fade_prepare
	lda #$10
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	lda #$20
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	lda #$30
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	lda #$40
	sta param_a
	jsr render_fade_step
	jsr render_tick
	;jsr render_tick
	jsr render_off
	jsr render_fade_finish
	rts

render_fade_in:
	jsr render_fade_prepare
	lda #$40
	sta param_a
	jsr render_fade_step
	jsr render_on
	jsr render_tick
	;jsr render_tick
	lda #$30
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	lda #$20
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	lda #$10
	sta param_a
	jsr render_fade_step
	jsr render_tick
	jsr render_tick
	;jsr render_tick
	jsr render_fade_finish
	rts

latch_ppu_0000:
	lda $2002
	lda #0
	sta $2006
	sta $2006
	rts

latch_ppu_2000:
	lda $2002
	lda #$20
	sta $2006
	lda #$00
	sta $2006
	rts

; A = 4k CHR block to load into $2007
load_chr:
	clc
	adc #CHR_BANK
	sta $5FFA
	lda #$A0
	sta temp_word0+1
	lda #$00
	sta temp_word0+0
	ldy #0
	ldx #0
	:
		lda (temp_word0), Y
		sta $2007
		iny
		bne :-
		inc temp_word0+1
		inx
		cpx #16
		bcc :-
	rts

; A = 1k NAM block to load into $2007
load_nam:
	pha
	lsr
	lsr ; 4 to a bank
	clc
	adc #NAM_BANK
	sta $5FFA
	pla
	and #3
	asl
	asl
	clc
	adc #$A0
	sta temp_word0+1
	lda #$00
	sta temp_word0+0
	ldy #0
	ldx #0
	:
		lda (temp_word0), Y
		sta $2007
		iny
		bne :-
		inc temp_word0+1
		inx
		cpx #4
		bcc :-
	rts

load_pal:
	pha
	lda #<data_pal
	sta temp_word0+0
	lda #>data_pal
	sta temp_word0+1
	pla
	pha
	asl
	asl
	asl
	asl
	clc
	adc temp_word0+0
	sta temp_word0+0
	lda temp_word0+1
	adc #0
	sta temp_word0+1
	pla
	lsr
	lsr
	lsr
	lsr
	clc
	adc temp_word0+1
	sta temp_word0+1
	ldy #0
	:
		lda (temp_word0), Y
		sta gfx_pal+ 0, Y
		sta gfx_pal+16, Y ; use same pal data for both for now
		iny
		cpy #16
		bcc :-
	rts

data_pal:
.incbin "art/fp.pal"

;
; text
;

text:
.byte "  BUN               ",$22,$23,$24,$23," ",$25,$26,$2A,$2B,$2A,"  " ; 00
.byte "  DJ DIMEBACK  ADRENALINE RUSH  " ; 01
.byte "  DUSTHILLGUY   I'M SO GLAD...  " ; 02
.byte "  GYROUH        BENNY THE SEAL  " ; 03
.byte "  KEFFIE                MOLOCH  " ; 04
.byte "  MAAK         EMOTIONAL MAGIC  " ; 05
.byte "  MXSP                 HI-TECH  " ; 06
.byte "  OFFGAO         OTENBA TOMBOY  " ; 07
.byte "  ROLEMUSIC           YOKUSHUN  " ; 08
.byte "  RUSHJET1  THE GRAND IMBECILE  " ; 09
.byte "  TOM MILLER    ANCIENT MELODY  " ; 0A
.byte "  INFORMATION          ",$1D,"        " ; 0B
; information screen
.byte "       ART BY CARPETBONES       " ; 0C
.byte "        LABEL BY MUSHBUH        " ; 0D
.byte "    CART BY INFINITENESLIVES    " ; 0E
.byte "      CODE BY RAINWARRIOR       " ; 0F
.byte "    PRODUCE BY KEVIN FARADAY    " ; 10
.byte "          <><><><><><>          " ; 11
.byte "                                " ; 12
.byte "            A - PLAY ALL        " ; 13
.byte "            B - PLAY SINGLE     " ; 14
.byte "        START - PAUSE           " ; 15
.byte "       SELECT - CANCEL          " ; 16
.byte "                       ",$1D,"        " ; 17
; extra
.byte "  DUSTHILLGUY      I'M SO GLAD  " ; 18
.byte "     CHRIS HOOPER IS MY FRIEND  " ; 19
.byte "     DISQUALIFIED FOR 224HZ     " ; 1A



; copies 32 bytes from (temp_word0) to $2007
copy_text:
	ldy #0
	:
		lda (temp_word0), Y
		sta $2007
		iny
		cpy #32
		bcc :-
	rts

; A = string to write (from text above)
; Y = row to write it on
copy_text_row:
	pha
	pha
	; latch PPU address to row Y
	lda #$2002
	tya
	pha
	lsr
	lsr
	lsr
	ora #$20
	sta $2006
	pla
	asl
	asl
	asl
	asl
	asl
	sta $2006
	; load text string A into temp_word0
	pla
	asl
	asl
	asl
	asl
	asl
	clc
	adc #<text
	sta temp_word0+0
	lda #>text
	adc #0
	sta temp_word0+1
	pla
	lsr
	lsr
	lsr
	clc
	adc temp_word0+1
	sta temp_word0+1
	; draw it
	jmp copy_text

.macro TEXT_ROW at, ar
	lda #at
	ldy #ar
	jsr copy_text_row
.endmacro

;
; visuals
;

; sprites (X,Y,tile,attrib) end with 128

particle00: .byte    0,  -2,$02,$00, 128 ; coloured dot, palette 0,1,2,3
particle01: .byte    0,  -2,$03,$00, 128 ; white dot
particle02: .byte    0,  -4,$04,$00, 128 ; white cross
particle03: .byte    0,  -4,$05,$00, 128 ; white open cross
particle04: .byte    0,  -8,$06,$02, 128 ; star, palette 2,3
particle05: .byte    0, -14,$07,$02      ; big star, palette 2,3
            .byte    5, -14,$07,$42
            .byte    0,  -6,$17,$02
            .byte    5,  -6,$17,$42, 128
particle06: .byte    0,  -8,$10,$00, 128 ; white shiny 0
particle07: .byte    0,  -8,$11,$00, 128 ; white shiny 1
particle08: .byte    0, -26,$20,$00      ; bubble huge, palette 0,1,2,3
            .byte    8, -26,$24,$00
            .byte    9, -26,$24,$40
            .byte   17, -26,$21,$40
            .byte    0, -18,$22,$00
            .byte   17, -18,$23,$40
            .byte    0, -10,$26,$80
            .byte    8,  -9,$24,$80
            .byte    9,  -9,$24,$C0
            .byte   17, -10,$28,$00, 128
particle09: .byte    0, -24,$30,$00      ; bubble large, palette 0,1,2,3
            .byte    8, -24,$32,$00
            .byte   15, -24,$31,$40
            .byte    0, -16,$33,$00
            .byte   15, -16,$33,$40
            .byte    0,  -8,$34,$80
            .byte    8,  -8,$35,$80
            .byte   15,  -8,$34,$C0, 128
particle0A: .byte    0, -16,$40,$00      ; bubble medium, palette 0,1,2,3
            .byte    7, -16,$41,$40
            .byte    0,  -8,$42,$80
            .byte    7,  -8,$42,$C0, 128
particle0B: .byte    0, -14,$50,$00      ; bubble small, palette 0,1,2,3
            .byte    5, -14,$51,$40
            .byte    0,  -6,$52,$80
            .byte    5,  -6,$52,$C0, 128
particle0C: .byte    0, -16,$40,$00      ; star bubble, palette 0,1,2,3
            .byte    7, -16,$41,$40
            .byte    0,  -8,$42,$80
            .byte    7,  -8,$42,$C0
            .byte    4, -13,$06,$02, 128
particle0D: .byte    0,  -8,$70,$00, 128 ; tiny bubble, palette 0,1,2,3

mascot:     .byte   120, 74,$60,$00      ; face mask for pico mascot
            .byte   132, 70,$61,$00
            .byte   125, 80,$62,$00, 128
valve:      .byte   160, 34,$63,$00      ; valve on pipe
            .byte   167, 34,$63,$40
            .byte   160, 41,$63,$80
            .byte   167, 41,$63,$C0, 128
slash:      .byte     0, -1,$E0,$22      ; track indicator slash
            .byte     8, -1,$E1,$22
            .byte    16, -1,$E2,$22
            .byte    24, -1,$E2,$22
            .byte    32, -1,$E2,$22
            .byte    40, -1,$E2,$22
            .byte    48, -1,$E3,$22
            .byte    56, -1,$E4,$22, 128

; control tables for palettes

particle_palette_table:
;      00   01   02   03   04   05   06   07   08   09   0A   0B   0C,  0D
.byte $23, $20, $20, $20, $21, $21, $20, $20, $23, $23, $23, $23, $23, $23
PARTICLE_PALETTE_TABLE_SIZE = * - particle_palette_table

particle_sprite_table:
.word particle00, particle01, particle02, particle03
.word particle04, particle05, particle06, particle07
.word particle08, particle09, particle0A, particle0B
.word particle0C, particle0D
PARTICLE_SPRITE_TABLE_SIZE = (* - particle_sprite_table) / 2

particle_table:
.byte $00, $00, $00, $01
.byte $02, $03, $04, $05
.byte $09, $0A, $0B, $0C
.byte $0D, $00, $00, $01
;.byte $06, $07, $08, $09 ; don't use white shiny or bubble huge

PARTICLE_TABLE_SIZE = * - particle_table
PARTICLE_TABLE_MASK = 1 << 4

.assert PARTICLE_PALETTE_TABLE_SIZE = PARTICLE_SPRITE_TABLE_SIZE, error, "particle table size mismatch"
.assert PARTICLE_TABLE_MASK >= PARTICLE_TABLE_SIZE, error, "particle table mask too small"
.assert (PARTICLE_COUNT & (PARTICLE_COUNT-1)) = 0, error, "PARTICLE_COUNT must be power of 2"

; visual code

prng:
	lda prng_seed+0
	asl
	rol prng_seed+1
	bcc :+
		eor #$BD
	:
	sta prng_seed+0
	rts

prng8:
	jsr prng
	jsr prng
	jsr prng
	jsr prng
	jsr prng
	jsr prng
	jsr prng
	jsr prng
	lda prng_seed+0
	rts

; X = particle
particle_spawn:
	; particle type
	jsr prng8
	:
		and #(PARTICLE_TABLE_MASK-1)
		cmp #PARTICLE_TABLE_SIZE
		bcc :+
		jsr prng
		jmp :-
	:
	tay
	lda particle_table, Y
	sta particle_t, X
	tay
	; X
	jsr prng8
	sta particle_x, X
	; Y
	lda #255
	sta particle_y, X
	; random palette selection
	jsr prng
	jsr prng
	jsr prng
	and particle_palette_table, Y
	sta particle_p, X
	; speed
	jsr prng8
	sta particle_s, X
	; if X is in middle, force attrib behind
	lda particle_x, X
	cmp #(115-23)
	bcc :+
	cmp #175
	bcs :+
		lda particle_p, X
		ora #$20
		sta particle_p, X
	:
	rts

wobble_06_07:
	lda particle_x, X
	clc
	adc nmi_count
	and #15
	bne :+
		lda particle_t, X
		eor #$01
		sta particle_t, X
	:
	lda particle_t, X
	rts

; X = particle
particle_tick:
	lda particle_s, X
	cmp speed_rng
	bcc :+
		dec particle_y, X
		beq particle_spawn
	:
	dec particle_y, X
	lda particle_y, X
	cmp #7 ; prevent large bubble hanging over to bottom of screen
	bcc particle_spawn
	lda particle_t, X
	;cmp #$06
	;beq :+
	;cmp #$07
	;beq :+
	;jmp :++
	;:
	;	jsr wobble_06_07
	;:
	asl
	tay
	lda particle_sprite_table+0, Y
	sta temp_word0+0
	lda particle_sprite_table+1, Y
	sta temp_word0+1
	lda particle_x, X
	sta param_a
	lda particle_y, X
	sta param_b
	lda particle_p, X
	sta param_c
	txa
	pha
	jsr sprite_add
	pla
	tax
	rts

particles_reset:
	ldx #0
	:
		jsr particle_spawn
		jsr prng8
		sta particle_y, X
		inx
		cpx #PARTICLE_COUNT
		bcc :-
	; setup special palette
	lda #$0F
	sta gfx_pal+(4*4)+1 ; black for mascot eyes
	lda #$21
	sta gfx_pal+(6*4)+1 ; rainbow for star
	lda #$37
	sta gfx_pal+(7*4)+1 ; rainbow for star
	rts

rainbow_hue:
	lda gfx_pal, X
	pha
	and #$30
	sta gfx_pal, X ; preserve intensity
	pla
	and #$0F
	clc
	adc #1
	cmp #$0D
	bcc :+
		lda #$01
	:
	ora gfx_pal, X
	sta gfx_pal, X
	rts

tick_add: ; randomize tick order by incrementing by prime
.byte 1, 15, 3, 13, 7, 5, 11, 17

particles_tick:
	; cycle star rainbows
	lda nmi_count
	and #3
	bne @rainbow_end
		ldx #(6*4)+1
		jsr rainbow_hue
		ldx #(7*4)+1
		jsr rainbow_hue
	@rainbow_end:
	; bit reverse frame count for sub-pixel speed faking
	lda #0
	sta speed_rng
	lda nmi_count
	ldx #8
	:
		ror
		rol speed_rng
		dex
		bne :-
	; tick the particles in randomized order
	inc tick_cycle
	lda tick_cycle
	and #7
	sta tick_cycle
	lda #0
	:
		pha
		tax
		jsr particle_tick
		ldx tick_cycle
		pla
		clc
		adc tick_add, X
		and #(PARTICLE_COUNT-1)
		bne :-
	rts

sprite_begin:
	lda #$FF
	ldx #0
	:
		sta oam, X
		inx
		bne :-
	lda #4 ; skip sprite 0
	sta oam_pos
	rts

; param_a = X
; param_b = Y
; param_c = OR attribute
; temp_word0 = sprite
sprite_add:
	ldy #0
	ldx oam_pos
	bne :+
		rts ; OAM full
	:
		lda (temp_word0), Y
		iny
		cmp #128
		beq :+
		clc
		adc param_a
		sta oam+3, X
		lda (temp_word0), Y
		iny
		clc
		adc param_b
		sta oam+0, X
		lda (temp_word0), Y
		iny
		sta oam+1, X
		lda (temp_word0), Y
		ora param_c
		iny
		sta oam+2, X
		inx
		inx
		inx
		inx
		stx oam_pos
		beq :+ ; OAM full
		jmp :-
	:
	rts

first_sprites_title:
	lda #0
	sta param_a
	sta param_b
	sta param_c
	lda #<mascot
	sta temp_word0+0
	lda #>mascot
	sta temp_word0+1
	jsr sprite_add
	lda #0
	sta param_a
	sta param_b
	sta param_c
	lda #<valve
	sta temp_word0+0
	lda #>valve
	sta temp_word0+1
	jsr sprite_add
	rts

first_sprites_main:
	lda #0
	sta param_a
	lda #-48
	sta param_b
	lda #0
	sta param_c
	lda #<mascot
	sta temp_word0+0
	lda #>mascot
	sta temp_word0+1
	jsr sprite_add
	rts

; A = tile
; dx/dy can be address or immediate(#)
; da attribute
.macro SpriteTile dx,dy,da
	pha
	ldx oam_pos
	lda dy
	sta oam, X
	inx
	pla
	sta oam, X
	inx
	lda da
	sta oam, X
	inx
	lda dx
	sta oam, X
	inx
	stx oam_pos
.endmacro

TIMER_X = (15 * 8) + 0
TIMER_Y = (19 * 8) - 1

timer_add:
	; minutes
	lda min1
	beq :+
		ora #$F0
		SpriteTile #TIMER_X-16,#TIMER_Y,#1
	:
	lda min0
	ora #$F0
	SpriteTile #TIMER_X-8,#TIMER_Y,#1
	; :
	lda #$FA
	SpriteTile #TIMER_X,#TIMER_Y,#0
	; seconds
	lda sec1
	ora #$F0
	SpriteTile #TIMER_X+8,#TIMER_Y,#1
	lda sec0
	ora #$F0
	SpriteTile #TIMER_X+16,#TIMER_Y,#1
	; play indicator
	lda play
	beq :+
		lda play_through
		clc
		adc #$FB ; play or play_through
	jmp :++
	:
		lda #$FD
	:
	SpriteTile #TIMER_X+4,#TIMER_Y+11,#2
	rts

;
; screens
;

init_title:
	lda #0
	jsr load_pal
	jsr latch_ppu_0000
	lda #0
	jsr load_chr
	lda #1
	jsr load_chr
	lda #0
	jsr load_nam
	jsr particles_reset
tick_title:
	jsr sprite_begin
	jsr first_sprites_title
	jsr particles_tick
	rts

init_main:
	jsr latch_ppu_2000
	lda #1
	jsr load_nam
	jsr particles_reset
	rts

init_tracks:
	jsr init_main
	.repeat TRACK_COUNT, I
		ldx #I
		lda track_order, X
		ldy #I+13
		jsr copy_text_row
	.endrepeat
	TEXT_ROW $0B, 24 ; INFORMATION
tick_tracks:
	jsr sprite_begin
	jsr first_sprites_main
	; track indicator
	lda slash_slider
	sta param_a ; X
	clc
	adc #23
	sta slash_slider
	lda track
	asl
	asl
	asl
	clc
	adc #(13*8)
	sta param_b ; Y
	lda #0
	sta param_c ; attribute OR
	lda #<slash
	sta temp_word0+0
	lda #>slash
	sta temp_word0+1
	jsr sprite_add
	; particles
	jsr particles_tick
	rts

init_info:
	jsr init_main
	.repeat 12, I
		TEXT_ROW I+$0C, I+13
	.endrepeat
tick_info:
	jsr sprite_begin
	jsr first_sprites_main
	jsr particles_tick
	rts

init_screen:
	;
	lda #0
	sta quad_hack
	; setup screen
	jsr init_main
	ldx track
	lda track_order, X
	cmp #$01
	bne :+
		; HACK track #$01 was written at 224 Hz
		; special playback hack for 4x speed (~240hz)
		lda #1
		sta quad_hack
		lda #$1A ; disqualified
		ldy #17
		jsr copy_text_row
		lda #$01
	:
	cmp #$02
	bne :+
		lda #$19 ; rest of dusthillguy title
		ldy #17
		jsr copy_text_row
		lda #$18 ; start of dusthillguy title
	:
	ldy #16
	jsr copy_text_row
tick_screen:
	jsr sprite_begin
	jsr first_sprites_main
	jsr timer_add
	jsr particles_tick
	.if PROFILE
		lda #%00011111
		sta $2001
	.endif
	rts

;
; blip sound
;

BLIP_LEN = 6
BLIP_LEN_CHEAT = 6 + 18
blip_freq: .byte $20, $30, $40, $50, $60, $70, $80, $F0,$F0,$F0,$D0,$D0,$D0,$E0,$E0,$E0,$C0,$C0,$C0,$A0,$A0,$A0,$B0,$B0,$B0
blip_vol:  .byte $B0, $B1, $B1, $B2, $B3, $B4, $B2, $35,$35,$35,$35,$35,$35,$75,$75,$75,$75,$75,$75,$B5,$B5,$B5,$B5,$B5,$B5

tick_blip:
	lda menu_blip
	bne :+ ; no blip if zero
		lda #0 ; mute
		sta $4015
		rts
	:
	cmp #BLIP_LEN_CHEAT
	beq @start
	cmp #BLIP_LEN
	bne @not_start
	@start:
		; start up sound on first frame of blip
		lda #%00000001 ; enable square 1
		sta $4015
		;
		lda blip_vol + BLIP_LEN ; volume
		sta $4000
		lda #%00000111 ; sweep (disabled)
		sta $4001
		lda blip_freq + BLIP_LEN ; low frequency
		sta $4002
		lda #%00001000 ; start channel, set high freq to 0
		sta $4003
		dec menu_blip
		jmp @blip_end
	@not_start:
	: ; else just tick sound
		ldx menu_blip
		lda blip_vol, x
		sta $4000
		lda blip_freq, x
		sta $4002
		dec menu_blip
		; auto-stop for cheat
		lda menu_blip
		cmp #BLIP_LEN
		bne :+
			lda #0
			sta menu_blip
		:
	@blip_end:
	rts

make_blip:
	lda #BLIP_LEN
	sta menu_blip
	rts

make_blip_cheat:
	lda #BLIP_LEN_CHEAT
	sta menu_blip
	rts

off_blip:
	lda #0
	sta menu_blip
	sta $4015
	rts

.export prog_main
prog_main:
	jmp do_title

do_title:
	lda #0
	sta menu_pos
	jsr init_title
	;jsr render_on
	jsr render_fade_in
	lda #1
	sta menu_hold ; wait for gamepad release
	@loop:
		jsr tick_title
		jsr tick_blip
		jsr render_tick
		jsr gamepad_poll
		lda gamepad
		bne :+
			; no press, clear menu_hold
			lda #0
			sta menu_hold
			jmp @loop
		:
		lda menu_hold
		bne @loop
		lda #1
		sta menu_hold
		; first press this frame
		lda gamepad
		and #(PAD_A | PAD_B | PAD_START)
		beq @no_start
			;jsr render_off
			jsr render_fade_out
			jsr make_blip
			jmp do_tracks
		@no_start:
		jmp @loop
	;


do_tracks:
	lda #0
	sta cheat_code
	jsr init_tracks
	;jsr render_on
	jsr render_fade_in
	@loop:
		jsr tick_tracks
		jsr tick_blip
		jsr render_tick
		jsr gamepad_poll
		; one response per press
		lda gamepad
		bne :+
			lda #0
			sta menu_hold
			jmp @loop
		:
		lda menu_hold
		bne @loop
		lda #1
		sta menu_hold
		; first press this frame
		lda gamepad
		and #(PAD_A | PAD_B | PAD_START)
		beq @no_start
			lda track
			cmp #TRACK_COUNT+1 ; INFORMATION
			bne :+
				;jsr render_off
				jsr render_fade_out
				jsr make_blip
				jmp do_info
			:
			lda gamepad
			and #PAD_B
			beq :+
				lda #0
				jmp :++
			:
				lda #1
			:
			sta play_through
			jsr off_blip
			;jsr render_off
			jsr render_fade_out
			jmp do_track
		@no_start:
		lda gamepad
		and #PAD_SELECT
		beq @no_quit
			jsr off_blip
			;jsr render_off
			jsr render_fade_out
			jmp do_title
		@no_quit:
		lda gamepad
		and #PAD_U
		beq @no_up
			lda #0
			sta cheat_code
			dec track
			lda track
			cmp #TRACK_COUNT
			bne :+
				dec track
			:
			lda track
			bpl :+
				lda #TRACK_COUNT+1
				sta track
			:
			jsr make_blip
			jmp @loop
		@no_up:
		lda gamepad
		and #PAD_D
		beq @no_down
			lda #0
			sta cheat_code
			inc track
			lda track
			cmp #TRACK_COUNT
			bne :+
				inc track
			:
			lda track
			cmp #TRACK_COUNT+2
			bcc :+
				lda #$00
				sta track
			:
			jsr make_blip
			jmp @loop
		@no_down:
		lda gamepad
		and #(PAD_L | PAD_R)
		cmp #PAD_L
		bne @no_left
			lda cheat_code
			and #1
			bne :+
				inc cheat_code
			:
			jmp @loop
		@no_left:
		lda gamepad
		and #(PAD_L | PAD_R)
		cmp #PAD_R
		bne @no_right
			lda cheat_code
			and #1
			bne :+
				jmp @loop
			:
			inc cheat_code
			lda cheat_code
			cmp #12
			bcs :+
				jmp @loop
			:
			lda #0
			sta cheat_code
			lda #TRACK_COUNT
			sta track
			jsr make_blip_cheat
			jmp @loop
		@no_right:
		jmp @loop
	;

do_info:
	jsr init_info
	;jsr render_on
	jsr render_fade_in
	lda #0
	sta menu_pos
	lda #1
	sta menu_hold
	@loop:
		jsr tick_info
		jsr tick_blip
		jsr render_tick
		jsr gamepad_poll
		lda gamepad
		bne :+
			; no press, clear menu_hold
			lda #0
			sta menu_hold
			jmp @loop
		:
		lda menu_hold
		bne @loop
		lda #1
		sta menu_hold
		; first press this frame
		lda gamepad
		and #(PAD_A | PAD_B | PAD_START | PAD_SELECT)
		beq @no_start
			;jsr render_off
			jsr render_fade_out
			jsr make_blip
			jmp do_tracks
		@no_start:
		jmp @loop
	;

do_track:
	lda #0
	sta min0
	sta min1
	sta sec0
	sta sec1
	lda #1
	sta menu_hold
	jsr init_screen
	;jsr render_on
	jsr render_fade_in
	jsr $8006 ; player.s track_play
	; if main_exit == 2 continue to next track instead
	lda main_exit
	cmp #2
	bne :+
		jmp do_track
	:
	jmp do_tracks

; end of file
