;
; Player bank sits at $8000-9FFF
; Is swapped in when not playing the current NSF.
;

.include "common.inc"

.import sta_5FF8, sta_5FF9, sta_5FFA, sta_5FFB, sta_5FFC, sta_5FFD, sta_5FFE, sta_5FFF
.import main_loop
.import tick_screen
.importzp TRACK_COUNT

.export gamepad_poll
.export track_order

.segment "TRAMP_NMI"
	jmp nmi
.segment "TRAMP_RETURN"
	jmp main_loop_return
.segment "TRAMP_TRACK"
	jmp track_play
.segment "TRAMP_MAIN"
	jmp main

.segment "CODE"

nmi:
	lda gfx_ready
	beq @gfx_end
	; copy palette
	lda #$3F
	sta $2006
	lda #$00
	sta $2006
	ldx #0
	:
		lda gfx_pal, x
		sta $2007
		inx
		cpx #32
	bne :-
	; copy nametable
	lda gfx_latch+0
	sta $2006
	lda gfx_latch+1
	sta $2006
	ldx #0
	cpx gfx_len
	beq :++ ; no nametable to update
	:
		lda gfx_nmt, x
		sta $2007
		inx
		cpx gfx_len
	bne :-
	:
	; OAM DMA
	lda #0
	sta $2003
	lda #>oam
	sta $4014
	; reset scroll
	lda #0
	sta $2005
	sta $2005
	; enable rendering
	lda #%00011110
	sta $2001
	; mark gfx_ready false
	lda #0
	sta gfx_len
	sta gfx_ready
@gfx_end:
	lda player_on
	beq :+
		jsr tick_player
	:
	rts

; main

.import prog_main
main:
	jmp prog_main

main_loop_return:
	lda #0
	sta play
	sta player_on
	sta $4015
	; return
	lda #$FF
	sta $5FFF ; restore startup bank
	rts

;
; track playback
;

.include "music_tables.inc"

track_play:
	; reset clock and other variables
	lda #0
	sta main_exit
	sta frame
	sta sec0
	sta sec1
	sta min0
	sta min1
	sta play_seconds+0
	sta play_seconds+1
	; PAL/DENDY adjustment
	lda pal
	cmp #2 ; DENDY
	bne :+
		lda #1
		sta pal_adjust_on
		lda #0
		sta pal_nsf ; play as if NTSC
		jmp @pal_adjust_end
	:
	cmp #1 ; PAL
	bne :+
		ldx track
		lda track_order, x
		tax
		lda track_pal_adjust, x
		sta pal_adjust_on
		eor #1
		sta pal_nsf ; play as if PAL unless adjust is on
		jmp @pal_adjust_end
	:
	lda #0 ; NTSC
	sta pal_adjust_on
	sta pal_nsf
@pal_adjust_end:
	lda #0
	sta pal_adjust
	sta track_nsf
	; setup init/play
	ldx track
	lda track_order, x
	asl
	tax
	lda track_init_addr+0, x
	sta init_addr+0
	lda track_init_addr+1, x
	sta init_addr+1
	lda track_play_addr+0, x
	sta play_addr+0
	lda track_play_addr+1, x
	sta play_addr+1
	; setup banks
	ldx track
	lda track_order, x
	tax
	lda track_bank_offset, x
	sta bank_add
	ldx track
	lda track_order, x
	asl
	asl
	asl
	tax
	lda track_bank_start+0, x
	clc
	adc bank_add
	sta last_8000 ; instead of banking this page right now, store this for when we jmp main_loop
	lda track_bank_start+1, x
	clc
	adc bank_add
	sta last_9000 ; ditto
	lda track_bank_start+2, x
	jsr sta_5FFA
	lda track_bank_start+3, x
	jsr sta_5FFB
	lda track_bank_start+4, x
	jsr sta_5FFC
	lda track_bank_start+5, x
	jsr sta_5FFD
	lda track_bank_start+6, x
	jsr sta_5FFE
	lda track_bank_start+7, x
	jsr sta_5FFF
	; clear ZP for NSF
	lda #0
	ldx #0
	:
		sta $00, x
		inx
		cpx #$F0
	bne :-
	; clear RAM for NSF
	ldx #0
	:
		sta $200, x
		sta $300, x
		sta $400, x
		inx
	bne :-
	; clear $4011
	lda #0
	sta $4011
	; turn on the player
	lda #1
	sta play
	sta player_on
.ifdef NSF
	rts
.else
	; jump to RAM loop
	jmp main_loop
.endif

CLOCK_POS := $2000+5+(16*32)

tick_player:
	jsr gamepad_poll
	; one response per press
	lda gamepad
	bne :+
		lda #0
		sta menu_hold
		jmp @done_input
	:
	lda menu_hold
	beq :+
		jmp @done_input
	:
	lda #1
	sta menu_hold
	; select quits to menu
	lda gamepad
	and #(PAD_SELECT)
	beq :+
		lda #1
		sta main_exit
	:
	; start pauses audio
	lda gamepad
	and #(PAD_START)
	beq @press_start_end
		lda play
		beq @unpause ; pause and mute
			lda #0
			sta play
			sta $4015
			jmp @press_start_end
		@unpause: ; else unpause and unmute
			lda #1
			sta play
			lda #%00001111
			sta $4015
		;
	@press_start_end:
	; B = single track
	lda gamepad
	and #(PAD_B)
	beq :+
		lda #0
		sta play_through
	:
	; A = continuous
	lda gamepad
	and #(PAD_A)
	beq :+
		lda #1
		sta play_through
	:
	; left/up = decrement track
	lda gamepad
	and #(PAD_L | PAD_U)
	beq :+
		lda track
		beq :+
		dec track
		lda #2
		sta main_exit
	:
	; right/down = increment track
	lda gamepad
	and #(PAD_R | PAD_D)
	beq :+
		lda track
		cmp #(TRACK_COUNT-1)
		bcs :+
		inc track
		lda #2
		sta main_exit
	:
@done_input:
	; advance clock
	lda play
	beq @clock_end
	inc frame
	lda frame
	cmp fps
	beq :+
		bcc @clock_end ; if ! (frame > fps) don't tick seconds
	:
	lda #0
	sta frame
	; increment internal seconds counter
	inc play_seconds+0
	bne :+
		inc play_seconds+1
	:
	; increment clock
	inc sec0
	lda sec0
	cmp #10
	bne @clock_end
	lda #0
	sta sec0
	inc sec1
	lda sec1
	cmp #6
	bne @clock_end
	lda #0
	sta sec1
	inc min0
	lda min0
	cmp #10
	bne @clock_end
	lda #0
	sta min0
	inc min1
	lda min1
	cmp #10
	bne @clock_end
	lda #0
	sta min1
	; roll over to 00:00
@clock_end:
	lda main_exit
	bne @time_check_end ; already ending the track
	; check track timer for song stop/advance
	ldx track
	lda track_order, x
	asl
	tax
	lda track_time+1, x
	cmp play_seconds+1
	beq :+
		bcs @time_check_end ; track_time high > play_seconds high
		jmp @time_over ; track_time_high < play_seconds high
	: ; track_time_high == play_seconds_high
	lda track_time+0, x
	cmp play_seconds+0
	beq @time_over
	bcs @time_check_end
@time_over:
	ldx track
	lda track_order, x
	tax
	lda track_loop, x
	beq :+
		; looping: halt the track time to prevent eventual overflow
		lda play_through
		bne :+ ; stop/mute the track if moving on
		ldx track
		lda track_order, x
		asl
		tax
		lda track_time+0, x
		sta play_seconds+0
		lda track_time+1, x
		sta play_seconds+1
		jmp :++
	:
		; if not looping, stop/mute
		lda #0
		sta play
		sta $4015
	:
	lda track ; don't skip if last track
	cmp #(TRACK_COUNT-1)
	bcs :+
		lda play_through
		beq :+ ; if continuous play, advance track
		inc track
		lda #2
		sta main_exit
	:
@time_check_end:
	; draw/animate
	lda main_exit
	bne :+
		jsr tick_screen
	:
	; done draw
	lda #1
	sta gfx_ready
	rts

;
; INPUT
;

gamepad_poll:
	; latch the current controller state
	lda #1
	sta $4016
	lda #0
	sta $4016
	; store high bit in gamepad to mark end of read
	lda #%10000000
	sta gamepad
	; read 8 bits from controller port
	:
		lda $4016
		and #%00000011
		cmp #%00000001
		ror gamepad
	bcc :-
	; DPCM conflict may have corrupted first read, test again to make sure
@gamepad_reread:
	lda gamepad
	pha ; store previous read on the stack
	lda #1
	sta $4016
	lda #0
	sta $4016
	lda #%10000000
	sta gamepad
	:
		lda $4016
		and #%00000011
		cmp #%00000001
		ror gamepad
	bcc :-
	pla ; pop the first read to compare
	cmp gamepad
	bne @gamepad_reread
	rts

; end of file